<?php
App::uses('EvShippingAppModel', 'EvShipping.Model');
/**
 * Zone Model
 *
 * @property Country $Country
 * @property ZoneExtra $ZoneExtra
 * @property ShippingRule $ShippingRule
 */
class Zone extends EvShippingAppModel {

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'country_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
				'message' => 'Please select a country to link this Zone to'
			),
		),
		'name' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'message' => 'Please enter a name for this Zone'
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * belongsTo associations
	 *
	 * @var array
	 */
	public $belongsTo = array(
		'Country' => array(
			'className' => 'EvCountry.Country'
		)
	);

	/**
	 * hasMany associations
	 *
	 * @var array
	 */
	public $hasMany = array(
		'ZoneExtra' => array(
			'className' => 'EvShipping.ZoneExtra',
			'dependent' => true,
			'cascade' => true
		)
	);

	/**
	 * hasAndBelongsToMany associations
	 *
	 * @var array
	 */
	public $hasAndBelongsToMany = array(
		'ShippingRule' => array(
			'className' => 'EvShipping.ShippingRule',
			'joinTable' => 'ev_shipping_shipping_rules_zones'
		)
	);

	/**
	 * readForEdit - retrieve the zone extra data
	 *
	 * @param integer $id ID of row to edit
	 * @param array $query The db query array - can be used to pass in additional parameters such as contain
	 * @return array
	 */
	public function readForEdit($id, $query = array()) {
		$query['contain'][] = 'ZoneExtra';

		return parent::readForEdit($id, $query);
	}

	/**
	 * Called after each successful save operation.
	 *
	 * @param bool $created True if this save created a new record
	 * @param array $options Options passed from Model::save().
	 * @return void
	 * @link http://book.cakephp.org/2.0/en/models/callback-methods.html#aftersave
	 * @see Model::save()
	 */
	public function afterSave($created, $options = array()) {
		parent::afterSave($created, $options);

		// process any custom field deletions
		if (! empty($this->data[$this->alias]['ZoneExtraDeletes']['toDelete'])) {

			$ids = explode(',', $this->data[$this->alias]['ZoneExtraDeletes']['toDelete']);
			$ids = ArrayUtil::clearArray($ids);

			$this->ZoneExtra->deleteAll(
				array(
					'ZoneExtra.id' => $ids
				),
				true,
				true
			);
		}
	}

	/**
	 * get the zones and the zone extra data given the country Id
	 *
	 * @param 	int 	$countryId 	Country Id
	 * @return  array 	$zones
	 */
	public function getByCountry($countryId) {
		return $this->find(
			'all',
			array(
				'conditions' => array(
					'Zone.country_id' => $countryId
				),
				'contain' => array(
					'ZoneExtra'
				)
			)
		);
	}

	/**
	 * given the list of applicable zones and the address
	 * try to filter on an extras
	 *
	 * @param 	array 		$zones  		Zones as returned from getByCountry()
	 * @param 	array 		$address 		The address to search
	 * @return 	array|bool 	$filteredZones	The filtered array of applicable zones
	 */
	public function filterOnExtras($zones, $address) {
		$primary = array(); // for zones that have extras and get matched
		$secondary = array(); // for zones withj no extras (i.e. a bkup)

		foreach ($zones as $zone) {
			if (empty($zone['ZoneExtra'])) {
				$secondary[] = $zone;
				continue;
			}

			foreach ($zone['ZoneExtra'] as $zoneExtra) {
				if ($this->zoneExtraMatch($zoneExtra, $address)) {
					$primary[] = $zone;
				}
			}
		}

		if (! empty($primary)) {
			return $primary;
		}

		if (empty($primary) && ! empty($secondary)) {
			return $secondary;
		}

		return false;
	}

	/**
	 * given the zone extra row, check if it matches giben the address
	 *
	 * @param 	array 	$zoneExtra 	Zone Extra row from the db
	 * @param 	array 	$address 	The address to try and match
	 * @return 	bool
	 */
	public function zoneExtraMatch($zoneExtra, $address) {
		// first check the field exists
		if (! isset($address[$zoneExtra['field']])) {
			return false;
		}

		$searchField = $address[$zoneExtra['field']];
		$operator = $zoneExtra['operator'];
		$value = $zoneExtra['field_value'];
		$searchValue = str_replace('%', '', $value);

		if ($operator == 'LIKE') {
			// find where / if it has any wildcards

			if (strpos($value, '%') === 0) {
				// wildcard at the beginng - check endsWith
				$checkEndsWith = true;
			}

			if (strpos($value, '%', 1) === (strlen($value) - 1)) {
				// wildcard at the end - check startsWith
				$checkStartsWith = true;
			}

			$SearchString = \Stringy\Stringy::create($searchField, 'UTF-8');

			if (isset($checkEndsWith) && isset($checkStartsWith)) {
				// has both - search contains
				return $SearchString->contains($searchValue);
			} elseif (isset($checkEndsWith)) {
				// wildcard at the beginning
				return $SearchString->endsWith($searchValue);
			} elseif (isset($checkStartsWith)) {
				// wildcard at the end
				return $SearchString->startsWith($searchValue);
			}
		}

		// fallback, simple equals check
		if (strtolower($searchField) == strtolower($value)) {
			return true;
		}

		return false;
	}
}
